//**************************************************************************************************
//
//  This Software and Related Documentation are Proprietary to Ageia
//  Technologies, Inc.
//
//  Copyright 2003 Ageia Technologies, Inc. St. Louis, MO
//  Unpublished -
//  All Rights Reserved Under the Copyright Laws of the United States.
//
//  Restricted Rights Legend:  Use, Duplication, or Disclosure by
//  the Government is Subject to Restrictions as Set Forth in
//  Paragraph (c)(1)(ii) of the Rights in Technical Data and
//  Computer Software Clause at DFARS 252.227-7013.  Ageia
//  Technologies Inc.
//
// 	Utils.cpp
//
//	Collection of good tools that do not belong in any class.
//
//**************************************************************************************************
#include "stdafx.h"
#include "Utils.h"

//**************************************************************************************************
//
//  getComputerName
//
//  Win32 API C++ wrapper for GetComputerName
//
//**************************************************************************************************
CString getComputerName()
{
    CString Name;
    char    name[ 100 ];
    DWORD   size = 100;

    GetComputerName( name, &size);
    Name.Format( "%s", name );

    return Name;
}

//**************************************************************************************************
//
// 	htoi
//			
//	Converts Hex Data into binary.
//			
//**************************************************************************************************
int htoi( const char *cp )
{
 // Variables.
	unsigned int i = 0;
	char c;

 // Move past leading spaces or tabs
    while(( *cp == ' ' ) || ( *cp == '\t' ))
	{
		++cp;
	}

 // Convert the number
	c = *cp;

 // Loop until we are done.
	for(;;)
	{
     // Check for a number.
		if(( c >= '0' ) && ( c <= '9' ))
		{
  	     // This is a digit 0-9
			i = ( i * 16 ) + ( c - '0' );
		}
		else
		{
   	     // Convert lower case to upper case if neccessary
			if(( c >= 'a' ) && ( c <= 'z' ))
			{
				c -= 32;
			}

            if( c == 'X' ) {}
			else if(( c >= 'A' ) && ( c <= 'F' ))
			{
   		     // A valid hexadecimal alpha
				i = ( i * 16 ) + ( c - 'A' + 10 );
			}
			else
			{
   		     // Invalid digit.
				break;
			}
		}

		cp++;
		c = *cp;
	}

	return( i );
}

//**************************************************************************************************
//
// 	htoi64
//			
//	Converts Hex Data into 64bit data.
//			
//**************************************************************************************************
_int64 htoi64( const char *cp )
{
 // Variables.
	_int64 i = 0;
	char c;

 // Move past leading spaces or tabs
    while(( *cp == ' ' ) || ( *cp == '\t' ))
	{
		++cp;
	}

 // Convert the number
	c = *cp;

 // Loop until we are done.
	for(;;)
	{
     // Check for a number.
		if(( c >= '0' ) && ( c <= '9' ))
		{
  	     // This is a digit 0-9
			i = ( i * 16 ) + ( c - '0' );
		}
		else
		{
   	     // Convert lower case to upper case if neccessary
			if(( c >= 'a' ) && ( c <= 'z' ))
			{
				c -= 32;
			}

            if( c == 'X' ) {}
			else if(( c >= 'A' ) && ( c <= 'F' ))
			{
   		     // A valid hexadecimal alpha
				i = ( i * 16 ) + ( c - 'A' + 10 );
			}
			else
			{
   		     // Invalid digit.
				break;
			}
		}

		cp++;
		c = *cp;
	}

	return( i );
}
//**************************************************************************************************
//
// 	ProcessMessagePump
//			
//	Allows other messages to be processed.
//			
//**************************************************************************************************
void ProcessMessagePump()
{
	MSG msg;
	while( ::PeekMessage(&msg, NULL, 0, 0, PM_REMOVE ))
	{
		::TranslateMessage( &msg );
		::DispatchMessage( &msg );
	}
}

//**************************************************************************************************
//
// 	XSleep
//			
//	Sleeps while allowing other messages to be processed.
//			
//**************************************************************************************************
void XSleep( int DelayTime_msec )
{
	while( DelayTime_msec > 0 )
	{
		Sleep( 10 );
		DelayTime_msec = DelayTime_msec - 10;
		ProcessMessagePump();
	}
}

//**************************************************************************************************
//
//	ReadFileIntoMemory
//
//	Opens the Target File and Reads it into the Specified Buffer.
//
//	Note:  Memory is ALLOCATED Here, the caller must free the Memory when done with the Buffer.
//
//  NOTE: This has been modified for 1Meg file...
//
//**************************************************************************************************
BYTE* ReadFileIntoMemory( CString PathName, UINT* FileLength )
{
    // Variables.
	CFile	File;
	BYTE*	FileData = NULL;

    // Attempt to Open and Read the Entire File into RAM.
	TRY
	{
	    // Attempt to Open the File.
		if( File.Open( PathName, CFile::modeRead ))
		{
		    // Get the File Size.
			*FileLength = File.GetLength();
	
	        // Get Memory to hold the file.
			FileData = new BYTE[ *FileLength ];

		    // Make sure we got memory to hold or data.
			if( ! FileData )
			{
			 // No memory, abort.
				File.Close();
				TRACE( "Could not allocate File Buffer Memory to Read in '%s' Size %d\n", 
                    PathName, *FileLength );
				return NULL;
			}
	
	        // Read the file into memory.
			if( File.Read( FileData, *FileLength ) != *FileLength )
			{
			    // Free the Memory and Close the File.
				delete FileData;
				File.Close();
				TRACE( "Could not Read File '%s' Size %d\n", PathName, *FileLength );
				return NULL;
			}
		}
		else
		{
		    // Close the File.
			TRACE( "Could not Open File '%s'\n", PathName );
			return NULL;
		}
	}

	CATCH(CFileException, pEx)
	{
        // Simply show an error message to the user.
		pEx->ReportError();
        return NULL;
	}
	END_CATCH

    // We have read the file into the Buffer.
	return FileData;
}

//**************************************************************************************************
//
//	CView* EK_GetActiveView()
//
//**************************************************************************************************
CView* EK_GetActiveView()
{
	CWnd* pMainWnd = AfxGetMainWnd();
	ASSERT_VALID( pMainWnd );

	if( !pMainWnd->IsKindOf( RUNTIME_CLASS( CFrameWnd )))
	{
		return NULL;
	}

	return static_cast<CFrameWnd*>( pMainWnd )->GetActiveFrame()->GetActiveView();
}

//**************************************************************************************************
//
//	BrowseForFolder CALLBACK
//
//  Callback used specify the Initial Folder.
//
//**************************************************************************************************
int CALLBACK BrowseCallbackProc( HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM dwData )
{
	switch( uMsg )
	{
	case BFFM_INITIALIZED:
		SendMessage( hwnd, BFFM_SETSELECTION, TRUE, dwData );
		TRACE( "In CallBack %s\n", dwData );

	    break;

	case BFFM_SELCHANGED:
	    break;

	case BFFM_VALIDATEFAILED:
	    break;
	}

	return 0;
}

//**************************************************************************************************
//
//	BrowseForFolder
//
//  Displays the ShellBrowseFolder Dialog and returns the selected Folder.  If the Cancel Button is 
//  pressed, an Empty String is returned.
//
//**************************************************************************************************
CString	BrowseForFolder( HWND hWnd, CString Title, CString InitialFolder )
{
    // Variables.
	BROWSEINFO 		bi;
	TCHAR 			szPath[MAX_PATH] = {0};
	LPITEMIDLIST 	pidl = NULL;
	LPMALLOC 		pMalloc = NULL;
	char			CurrentDir[ MAX_PATH ];

    // Prepare the call
	ZeroMemory( &bi, sizeof( BROWSEINFO ));
	bi.hwndOwner = hWnd;

    // Add the Target Title.
	bi.lpszTitle = Title;

    // Set up the callback
	bi.lpfn = BrowseCallbackProc;
	strcpy( CurrentDir, InitialFolder );
	bi.lParam = (LPARAM)&CurrentDir;

    // Set up the flags.
	UINT uiFlags = 0;
	uiFlags |= BIF_RETURNONLYFSDIRS;
	bi.ulFlags = uiFlags;

    // Call the function
	LPITEMIDLIST pidlFolder = SHBrowseForFolder(&bi);
	SHGetPathFromIDList( pidlFolder, szPath );

    // Free
	SHGetMalloc(&pMalloc);
	pMalloc->Free(pidl);
	pMalloc->Free(pidlFolder);
	pMalloc->Release();

    // Return the Target Path.
	return( szPath );
}

//**************************************************************************************************
//
//	GetFileList
//
//  Returns a List of Files that Match the given filter( TargetPath)  The return can contain the 
//      FullPath or Just the FileNames.
//
//  NOTE: TragetPath: Null-terminated string that specifies a valid directory or path and filename, 
//        which can contain wildcard characters (* and ?). 
//
//**************************************************************************************************
BOOL GetFileList( CString TargetPath, BOOL FullPath, CStringArray* FileList )
{
    // Get the File List from the Disk.
	WIN32_FIND_DATA FileData;
	HANDLE 		    hSearch;
    CString         FullPathName;

    // Build the FullPathName.
    FullPathName.Format( "%s%s", GetDrive( TargetPath ), GetDirectory( TargetPath ));

    // Find the first file.
	hSearch = FindFirstFile( TargetPath, &FileData );
	if( hSearch == INVALID_HANDLE_VALUE )
	{
        // Close the search handle
        FindClose( hSearch );

        // An error occurred.
        TRACE( "GetFileList Filaure: FindFirstFile() '%s'\n", TargetPath );
		return FALSE;
	}
	
    // Add the First File.
    if( FileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY )
    {
     // These are directories.

    }
    else
    {
        // Do we want FullPath Names.
        if( FullPath )
        {
            // Add the Full PathName.
            FileList->Add( FullPathName + FileData.cFileName );
        }
        else
        {
            // Only add the FileName.
            FileList->Add( FileData.cFileName );
        }
    }

    // Search through the rest of the files.
    while( 1 )
    {
        // Are we at the end.
        if( !FindNextFile( hSearch, &FileData ))
        {
            // Close the search handle.
            if( !FindClose( hSearch ))
            {
                // Something failed.
                TRACE( "GetFileList Filaure: FindNextFile() '%s'\n", TargetPath );
                return FALSE;
            }

         // Debug: Dump the FileList.
         #if 0
            for( int Index = 0; Index < FileList->GetSize(); Index++ )
            {
                TRACE( "File: %d, '%s'\n", Index, FileList->GetAt( Index ));
            }
         #endif 

            // All is OK.
            return TRUE;
        }
        else
        {
            // Add this file to the list.  Make sure it is not the Personal Dictionary.
            if( FileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY )
            {
                // These are directories.
            }
            else
            {
                // Do we want FullPath Names.
                if( FullPath )
                {
                    // Add the Full PathName.
                    FileList->Add( FullPathName + FileData.cFileName );
                }
                else
                {
                    // Only add the FileName.
                    FileList->Add( FileData.cFileName );
                }
            }
        }
    }
}

//**************************************************************************************************
//
// 	GetDrive
//			
//  Retrieves the Drive String from a Path Name.
//			
//	Sample:  'c:\sample\crt\makepath.c'
//	Returns: 'c:'
//			
//**************************************************************************************************
CString GetDrive( CString Path )
{
    // Variables.
	char drive[ _MAX_DRIVE ];
	char dir[   _MAX_DIR ];
	char fname[ _MAX_FNAME ];
	char ext[   _MAX_EXT ];

	_splitpath( Path, drive, dir, fname, ext );

	return( (CString)drive );
}

//**************************************************************************************************
//
// 	GetDirectory
//			
//  Retrieves the Directory String from a Path Name.
//
//	Sample:  'c:\sample\crt\makepath.c'
//	Returns: '\sample\crt\'
//			
//**************************************************************************************************
CString GetDirectory( CString Path )
{
    // Variables.
	char drive[ _MAX_DRIVE ];
	char dir[   _MAX_DIR ];
	char fname[ _MAX_FNAME ];
	char ext[   _MAX_EXT ];

	_splitpath( Path, drive, dir, fname, ext );

	return( (CString)dir );
}

//**************************************************************************************************
//
// 	GetFile
//			
//  Retrieves the File String from a Path Name.
//
//	Sample:  'c:\sample\crt\makepath.c'
//	Returns: 'makepath'
//			
//**************************************************************************************************
CString GetFile( CString Path )
{
    // Variables.
	char drive[ _MAX_DRIVE ];
	char dir[   _MAX_DIR ];
	char fname[ _MAX_FNAME ];
	char ext[   _MAX_EXT ];

	_splitpath( Path, drive, dir, fname, ext );

	return( (CString)fname );
}

//**************************************************************************************************
//
// 	GetExt
//			
//  Retrieves the File Extension from a Path Name.
//
//	Sample:  'c:\sample\crt\makepath.c'
//	Returns: 'c'
//			
//**************************************************************************************************
CString GetExt( CString Path )
{
    // Variables.
	char drive[ _MAX_DRIVE ];
	char dir[   _MAX_DIR ];
	char fname[ _MAX_FNAME ];
	char ext[   _MAX_EXT ];

	_splitpath( Path, drive, dir, fname, ext );

	return( (CString)ext );
}

//**************************************************************************************************
//
// 	GetCurrentPathName
//			
//  Retrieves the current working directory for the current process.
//			
//**************************************************************************************************
CString GetCurrentPathName()
{
	char WorkingPath[ MAX_PATH ];
	GetCurrentDirectory( MAX_PATH, WorkingPath );						

    return( (CString)WorkingPath );
}

//**************************************************************************************************
//
// 	SetCurrentPathName
//			
//  Changes the current directory for the current process.
//			
//**************************************************************************************************
void SetCurrentPathName( CString PathName )
{
	SetCurrentDirectory( PathName );						
}

//**************************************************************************************************
//
// 	GetVolumeSerialNumber
//			
//	Gets the Volume Serial Number from the Hard Drive "C:\"
//			
//**************************************************************************************************
int	GetVolumeSerialNumber()
{
    // Variables.
    char 	VolumeName[ 100 ];
    DWORD 	VolumeNumber;
    DWORD 	Dummy1, Dummy2;
    char 	Dummy[ 100 ];

    // Get the Hard Drive Information.
    GetVolumeInformation( "C:\\", VolumeName, 100, &VolumeNumber, &Dummy1, &Dummy2, Dummy, 100 );

    // Return the Volume Serial Number.
	return( VolumeNumber );
}

//**************************************************************************************************
//
//	DoesFileExsist
//
//  Checks if the given PathName currently exsist on the disk.
//
//**************************************************************************************************
BOOL DoesFileExsist( CString PathName )
{
    // Get the file attributes.
	if( GetFileAttributes( PathName ) == -1 )
	{
	    // Get the error code.
        if( GetLastError() == ERROR_FILE_NOT_FOUND )
		{	
			TRACE( "DoesFileExsist '%s' FALSE\n", PathName );
			return( FALSE );
		}
	}	

    // If we make it this far, the exsist.
	TRACE( "DoesFileExsist '%s' TRUE\n", PathName );
	return( TRUE );
}

//**************************************************************************************************
//
//	ShellCopyFile (OsShell Helper)
//
//	Uses the Shell to Copy a File.
//
//**************************************************************************************************
BOOL ShellCopyFile( HWND phWnd, CString To, CString From, FILEOP_FLAGS MoreFlags )
{
    // Variables.
	SHFILEOPSTRUCT	shfo;
    int				len;
	char			from[ MAX_PATH ];

    // Make sure we double NULL.
	strcpy( from, From );
	len = strlen( from );
	from[ len + 1 ] = 0;

	TRACE( "ShellCopyFile To: '%s' From: '%s'\n", To, from );

    // Initializae the FileStruct.
	ZeroMemory( &shfo, sizeof( SHFILEOPSTRUCT ));
	shfo.hwnd   = phWnd;
	shfo.wFunc  = FO_COPY;
	shfo.fFlags = FOF_FILESONLY | MoreFlags;
	shfo.pTo    = To;
	shfo.pFrom  = from;

    // Make the call.
	SHFileOperation( &shfo );

    // Return.
	return TRUE;
}

//**************************************************************************************************
//
//	SendFileToRecycleBin (OsShell Helper)
//
//	Will send the given file to the Recycle Bin using the Shell
//
//**************************************************************************************************
int	SendFileToRecycleBin( HWND phWnd, CString* FileName, BOOL Recycle )
{
    // Variables.
	SHFILEOPSTRUCT	shfo;
	int				len;
	char			FileNames[ MAX_PATH ];

    // Format the FileNames Buffer.  We need this to insure we have a Double NULL.
	sprintf( FileNames, "%s", *FileName );
	len = strlen( FileNames );
	FileNames[ len + 1 ] = 0;

	TRACE( "SendFileToRecycleBin '%s'\n", FileNames );

    // Initializae the FileStruct.
	ZeroMemory( &shfo, sizeof( SHFILEOPSTRUCT ));
	shfo.hwnd   = phWnd;
	shfo.wFunc  = FO_DELETE;
	shfo.fFlags = FOF_FILESONLY;
	shfo.pFrom  = FileNames;

    // Check the Recycle Flag.
	if( Recycle )
	{
        // Add the Undo to the Flags.
		shfo.fFlags |= FOF_ALLOWUNDO;
	}
	else
	{
        // We are not recycling( Deleting Only ), do not show the UI.
		shfo.fFlags |= FOF_NOCONFIRMATION;
	}

    // Make the call.
	return SHFileOperation( &shfo );
}
